import Position from '../position/Position'

const {Cesium} = BC.Namespace

class Water {
  constructor(pos, opt) {
    if (!(pos instanceof Position)) {
      console.error('参数缺失，水的起点坐标，请new BC.Position（)')
      pos = new Position()
    }
    this.position = pos
    this.opt = opt
    this.entity = null
    this.posEntity = null
    Cesium = BC.Namespace.Cesium
  }

  init() {
    if (!window.viewer) {
      console.error('地球未初始化，水流加载失败')
    }
    this.posEntity = viewer.entities.add({
      position: Cesium.Cartesian3.fromDegrees(
        this.position.lng,
        this.position.lat,
        this.position.alt
      )
    })
    this.entity = viewer.scene.primitives.add(
      new Cesium.ParticleSystem({
        image: this.opt?.image || '/libs/resources/static/water.png',
        startColor: Cesium.Color.fromBytes(
          this.opt?.startColor?.[0] || 255,
          this.opt?.startColor?.[1] || 255,
          this.opt?.startColor?.[2] || 255,
          this.opt?.startColor?.[3] * 255 || 255
        ),
        endColor: Cesium.Color.fromBytes(
          this.opt?.endColor?.[0] || 255,
          this.opt?.endColor?.[1] || 255,
          this.opt?.endColor?.[2] || 255,
          this.opt?.endColor?.[3] * 255 || 255
        ),
        startScale: this.opt?.startScale || 0.5,
        endScale: this.opt?.endScale || 3,
        sizeInMeters: this.opt?.sizeInMeters || true,
        minimumParticleLife: this.opt?.minimumParticleLife || 1,
        maximumParticleLife: this.opt?.maximumParticleLife || 10,
        speed: this.opt?.speed || 26,
        mass: this.opt?.mass || 1,
        imageSize: new Cesium.Cartesian2(
          this.opt?.imageSize?.width || 2,
          this.opt?.imageSize?.height || 2
        ),
        emissionRate: this.opt?.emissionRate || 200,
        lifetime: this.opt?.lifetime || 16.0,
        emitter: new Cesium.CircleEmitter(this.opt?.emitter || 0.1),
        emitterModelMatrix: this.computeEmitterModelMatrix(),
        updateCallback: this.applyGravity
      })
    )
  }

  computeEmitterModelMatrix() {
    var trs = new Cesium.TranslationRotationScale()
    var hpr = Cesium.HeadingPitchRoll.fromDegrees(
      this.position.heading,
      this.position.pitch,
      this.position.roll,
      new Cesium.HeadingPitchRoll()
    ) // 倾斜角度
    trs.translation = Cesium.Cartesian3.fromElements(
      0,
      0,
      1,
      new Cesium.Cartesian3()
    ) // 发射高度
    trs.rotation = Cesium.Quaternion.fromHeadingPitchRoll(
      hpr,
      new Cesium.Quaternion()
    )
    return Cesium.Matrix4.fromTranslationRotationScale(
      trs,
      new Cesium.Matrix4()
    )
  }

  computeModelMatrix(entity, time) {
    return entity.computeModelMatrix(time, new Cesium.Matrix4())
  }

  applyGravity(p, dt) {
    var gravityScratch = new Cesium.Cartesian3()
    var position = p.position
    Cesium.Cartesian3.normalize(position, gravityScratch)
    Cesium.Cartesian3.multiplyByScalar(gravityScratch, -4 * dt, gravityScratch)
    p.velocity = Cesium.Cartesian3.add(p.velocity, gravityScratch, p.velocity)
  }

  update(scene, time) {
    this.entity.modelMatrix = this.computeModelMatrix(this.posEntity, time)
    this.entity.emitterModelMatrix = this.computeEmitterModelMatrix()
  }

  start() {
    if (this.entity) {
      this.entity.show = true
      return
    }
    this.init()
    viewer.scene.preUpdate.addEventListener(this.update.bind(this))
  }

  stop() {
    if (!this.entity) return
    this.entity.show = false
    viewer.scene.preUpdate.removeEventListener(this.update.bind(this))
  }

  uninstall() {
    if (!this.entity) return
    viewer.scene.primitives.remove(this.entity)
    viewer.scene.preUpdate.removeEventListener(this.update.bind(this))
    this.entity = null
  }
  zoomTo() {
    viewer.zoomTo(this.entity)
  }
}

export default Water
